#ifndef __HTTPD_H__
#define __HTTPD_H__

/*
  Web Server


 Circuit:
 * Ethernet shield attached to pins 10, 11, 12, 13

This functionality inspired by Mini-Circuits rf-switch, which respons to:
requests.get("http://%s/SP6T%s:STATE:%d" % (ip, switch, state))

 */

 
#include <SPI.h>
#include <Ethernet.h>
#include "rf_mux.h"

namespace HTTPD {
  


byte mac[] = {
//  0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xED
  0x90, 0xA2, 0xDA, 0x0E, 0xCE, 0x76

};

IPAddress ip(192, 168, 1, 18);
//IPAddress ip(192, 168, 3, 30);
//IPAddress ip(130, 188, 45, 253);

class httpd {
  
public:

httpd(RF_MUX::RF_MUX* m1, RF_MUX::RF_MUX* m2){
  server = new EthernetServer(80); // (port 80 is default for HTTP):
  _m1 = m1;
  _m2 = m2;
}
~httpd(){
  delete server;
}

void init() {
  // start the Ethernet connection and the server:
  Ethernet.begin(mac, ip); // fixed IP
  //Serial.println("DHCP...");
  //int dhcp_result = Ethernet.begin(mac); // DHCP result
  //Serial.print("DHCP success? ");
  //Serial.println(dhcp_result);
  server->begin();
  Serial.print("MUX is at IP: ");
  Serial.println(Ethernet.localIP());
  _m1->init();
  _m2->init();
}

void available() {
  // listen for incoming clients
  EthernetClient client = server->available();
  if (client) {
    //Serial.println("new client");
    // an http request ends with a blank line
    boolean currentLineIsBlank = true;
    while (client.connected()) {
      if (client.available()) {
        char c = client.read();
        _httpRequest += c;
        // Serial.write(c);
        
        // if you've gotten to the end of the line (received a newline
        // character) and the line is blank, the http request has ended,
        // so you can send a reply
        if (c == '\n' && currentLineIsBlank) {
          // send a standard http response header
          _httpRequest.toUpperCase();
          //Serial.print("_httpRequest: ");
          //Serial.println( _httpRequest);
          if (_httpRequest.indexOf("GET / ") >= 0) {
              client.println("HTTP/1.1 200 OK");
              client.println("Content-Type: text/html");
              client.println("Connection: close");  // the connection will be closed after completion of the response
              //client.println("Refresh: 5");  // refresh the page automatically every 5 sec
              client.println();
              client.println("<!DOCTYPE HTML>");
              client.println("<html>");
              //client.println("Index page");

              print_state(&client);

              client.println("</html>");
          }
          else if (_httpRequest.indexOf("GET /SP8T") >= 0){
              // parse command

              parse_command();
              
              client.println("HTTP/1.1 200 OK");
              client.println("Content-Type: text/html");
              client.println("Connection: close");  // the connection will be closed after completion of the response
              //client.println("Refresh: 5");  // refresh the page automatically every 5 sec
              client.println();
              client.println("<!DOCTYPE HTML>");
              client.println("<html>");
              
              
              print_state(&client);
              client.println("</html>");
          }

          _httpRequest = "";
          break; // request done- break out of while(connected)
          
        }
        if (c == '\n') { // you're starting a new line
          currentLineIsBlank = true;
        } else if (c != '\r') {           // you've gotten a character on the current line
          currentLineIsBlank = false;
        }
      }
    }
    // give the web browser time to receive the data
    delay(1);
    // close the connection:
    client.stop();
    //Serial.println("client disconnected");
  }
}// available()

#define DELIMETERS " :/" // split the httpRequest with these delimeters
void parse_command() {
  char *strings[10]; // SP8T:A:STATE:2   four tokens
  char *ptr = NULL;
  byte index = 0;
  char buf[ _httpRequest.length() +1 ];
  _httpRequest.toCharArray(buf, _httpRequest.length());
  //Serial.println("strtok() %s", buf);
  ptr = strtok(buf, DELIMETERS);
  while(ptr != NULL) {
        strings[index] = ptr;
        index++;
        ptr = strtok(NULL, DELIMETERS);  // takes a list of delimiters
        if (index>4) // stop parsing string when we found what we want
          break;
 }
 /*
   for(int n = 0; n < index; n++) {
    Serial.print("Token ");
    Serial.print(n);
    Serial.print(" ");
    Serial.println(strings[n]);
   }*/
   if ( strcmp(strings[0], "GET")==0 ) {
    if (strcmp(strings[1], "SP8T")==0 ) {
      RF_MUX::RF_MUX* mux;
      uint8_t state;
      if (strcmp(strings[2], "1") == 0) {
        mux = _m1;
      } else if ( strcmp(strings[2], "2")  == 0) {
        mux = _m2;
      } else {
        return;
      }
      if (strcmp(strings[3], "STATE") == 0) {
          //Serial.println("command received");
          //Serial.print("MUX ");
          //Serial.print(mux->get_id());
          uint8_t state = atoi( strings[4] );
          //Serial.print(" STATE ");
          //Serial.println(state);
          mux->set(state);
      }


   }
   }
}

void advance(int mux_id) {
  // move one step forward
  RF_MUX::RF_MUX* mux;
  if (mux_id==0) {
    mux = _m1;
  } else if (mux_id==1) {
    mux = _m2;
  }
  mux->advance();
  delay(1000);
  
}

void print_state(EthernetClient* client) {
    // www-browser sees this output.
    client->print("SP8T:");
    client->print( _m1->get_id() );
    client->print(":STATE:");
    client->print( _m1->get() );
    client->println("<BR>");
    
    client->print("SP8T:");
    client->print( _m2->get_id() );
    client->print(":STATE:");
    client->print( _m2->get() );
    client->println("<BR>");
} // print_state

private:
  EthernetServer* server;
  String _httpRequest;
  RF_MUX::RF_MUX* _m1;
  RF_MUX::RF_MUX* _m2;
}; // end class
} // end namespace
#endif 
