#ifndef __RF_MUX_H__
#define __RF_MUX_H__

#include "mcp23s17.h"

namespace RF_MUX {

class RF_MUX {
  public:

  RF_MUX(char id, int8_t SDIO, int8_t CSB, int8_t SCLK) {
    _io = new MCP23S17::MCP23S17(SDIO, CSB, SCLK);
    _id = id;
    _state = 0;
  }
  ~RF_MUX() {
    delete _io;
  }
  void init(){
    _io->set_pin_mode(MCP23S17::PINMODE_OUTPUT);
    _state=1; // to ensure we set zero below
    this->set(0); // reset to state zero
  }
  void set(uint8_t ch) {
    // don't accept invalid states
    if (ch<0) {
      return;
    } else if (ch>8) {
      return;
    } else if (ch==_state) {
      return; // nothing to do.
    }

    // valid state, do change:
    _state = ch;
    _io->set_gpio( MCP23S17::PINA[_state], MCP23S17::PINB[_state] );
    Serial.print("MUX ");
    Serial.print(_id);
    Serial.print(" state ");
    Serial.print(_state);
    Serial.print("\n");
  }
  void advance() {
    uint8_t newstate = this->get()+1;
    if (newstate>8) {
      newstate = 0;
    }
    this->set(newstate);
  }
  
  uint8_t get() {
    return _state;
  }
  const char get_id() {
    return _id;
  }
  private:
      MCP23S17::MCP23S17* _io;
      uint8_t _state;
      char _id;

}; // end class

} // end namespace

#endif 
